@@ -4,7 +4,7 @@ use strict;
 use warnings;
 
 use FTPServer;
-use WgetTest;
+use WgetTests;
 
 our @ISA = qw(WgetTest);
 my $VERSION = 0.01;
@@ -58,4 +58,3 @@ sub _substitute_port {
 1;
 
 # vim: et ts=4 sw=4
-
